//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Data;
using System.Drawing;
using System.IO;
using System.Windows.Forms;


namespace MindFusion.Diagramming.WinForms.Samples.CS.SWDesigner
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			_this = this;

			InitializeComponent();
		}

		private bool CloseProject()
		{
			if (_project != null)
			{
				// Save diagrams
				foreach (DiagramForm child in MdiChildren)
					child.Save();

				bool close = true;

				if (_project.IsModified)
				{
					DialogResult result = MessageBox.Show(
						"The current project has been modified. Save?",
						"Closing project", MessageBoxButtons.YesNoCancel,
						MessageBoxIcon.Question);

					switch (result)
					{
						case DialogResult.Yes:
							_project.Save();
							close = true;
							break;
						case DialogResult.No:
							close = true;
							break;
						case DialogResult.Cancel:
							close = false;
							break;
					}
				}

				if (!close)
					return close;

				// Close forms
				foreach (DiagramForm child in MdiChildren)
					child.Close();

				// Remove tree items
				TreeNode root = _tvProject.Nodes[0];
				root.Nodes.Clear();

				_project = null;
			}

			return true;
		}

		private Diagram SelectedDiagram
		{
			get
			{
				TreeNode node = _tvProject.SelectedNode;
				if (node == null)
					return null;
				if (node.Parent == null)
					return null;

				return (Diagram)node.Tag;
			}
		}


		private void _menuNew_Click(object sender, System.EventArgs e)
		{
			CreateProjectDialog dlg = new CreateProjectDialog();

			if (dlg.ShowDialog(this) == DialogResult.OK)
			{
				CloseProject();

				_project = new Project();
				_project.Create(ProjectFolder +
					Tools.FixFileName(dlg.ProjectTitle) + System.IO.Path.DirectorySeparatorChar,
					dlg.ProjectTitle);
			}
		}

		private void _menuOpen_Click(object sender, System.EventArgs e)
		{
			_dlgOpen.InitialDirectory = ProjectFolder;

			if (_dlgOpen.ShowDialog(this) == DialogResult.OK)
			{
				CloseProject();

				_project = new Project();
				_project.Load(
					System.IO.Path.GetDirectoryName(_dlgOpen.FileName) +
					System.IO.Path.DirectorySeparatorChar);

				// Initialize diagrams
				foreach (Diagram d in _project.Diagrams)
				{
					TreeNode node = new TreeNode(d.Name);
					node.Tag = d;
					_tvProject.Nodes[0].Nodes.Add(node);
				}
			}
		}

		private void _menuClose_Click(object sender, System.EventArgs e)
		{
			CloseProject();
		}

		private void _menuSave_Click(object sender, System.EventArgs e)
		{
			if (_project != null)
			{
				// Save all open diagrams
				foreach (DiagramForm child in MdiChildren)
					child.Save();

				_project.Save();
			}
		}

		private void _menuExit_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void _menuNewDiagram_Click(object sender, System.EventArgs e)
		{
			if (_project == null)
				return;

			AddDiagramDialog dlg = new AddDiagramDialog();

			if (dlg.ShowDialog(this) == DialogResult.OK)
			{
				// Check for unique name
				string unique = dlg.DiagramName;
				int index = 0;
				bool found = false;
				do
				{
					found = false;
					foreach (Diagram d in _project.Diagrams)
						if (d.Id == unique)
						{
							found = true;
							break;
						}

					if (!found)
						break;

					unique = dlg.DiagramName + index.ToString();
					index++;
				}
				while (found);

				Diagram diagram = new Diagram(dlg.DiagramName, unique);
				_project.AddDiagram(diagram);

				// Add to tree view
				TreeNode node = new TreeNode(diagram.Name);
				node.Tag = diagram;
				_tvProject.Nodes[0].Nodes.Add(node);

				// Display diagram
				_tvProject.SelectedNode = node;
				_tvProject_DoubleClick(null, null);
			}
		}

		private void _menuDelete_Click(object sender, System.EventArgs e)
		{
			Diagram diagram = SelectedDiagram;
			if (diagram == null)
				return;

			if (MessageBox.Show("Confirm delete?", "Delete diagram",
				MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
				return;

			foreach (DiagramForm child in MdiChildren)
				if (child.Diagram == diagram)
					child.Close();

			_tvProject.SelectedNode.Remove();
			_project.RemoveDiagram(diagram);
		}

		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			e.Cancel = !CloseProject();
		}

		private void _tvProject_DoubleClick(object sender, System.EventArgs e)
		{
			Diagram diagram = SelectedDiagram;
			if (diagram == null)
				return;

			// See if diagram is already open
			foreach (DiagramForm child in MdiChildren)
				if (child.Diagram == diagram)
				{
					child.Activate();
					return;
				}

			DiagramForm form = new DiagramForm();

			form.Diagram = diagram;
			form.Text = "Class diagram - [" + diagram.Name + "]";
			form.MdiParent = this;
			form.Show();
		}

		private void _cmenuRename_Click(object sender, System.EventArgs e)
		{
			Diagram diagram = SelectedDiagram;
			if (diagram == null)
				return;

			AddDiagramDialog dlg = new AddDiagramDialog();

			dlg.Text = "Rename diagram";
			dlg.DiagramName = diagram.Name;
			if (dlg.ShowDialog(this) == DialogResult.OK)
			{
				diagram.Name = dlg.DiagramName;

				// Check if form is open
				foreach (DiagramForm child in MdiChildren)
					if (child.Diagram == diagram)
					{
						child.Text = "Class diagram - [" + diagram.Name + "]";
						_tvProject.SelectedNode.Text = diagram.Name;
						break;
					}
			}
		}

		private void UpdateMenus()
		{
			_menuClose.Enabled = (_project != null);
			_menuDelete.Enabled = (_project != null);
			_menuAdd.Enabled = (_project != null);
			_menuSave.Enabled = (_project != null);
		}

		private void _tvProject_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter)
				_tvProject_DoubleClick(sender, e);
		}

		public static string ProjectFolder
		{
			get
			{
				return System.IO.Path.GetDirectoryName(Application.ExecutablePath) +
					System.IO.Path.DirectorySeparatorChar + "Projects" +
					System.IO.Path.DirectorySeparatorChar;
			}
		}


		private static MainForm _this;

		private Project _project;
	}

	public class Tools
	{
		public static string FixFileName(string name)
		{
			string result = name;
			int index = -1;

			while ((index = result.IndexOfAny(System.IO.Path.GetInvalidPathChars())) != -1)
				result.Remove(index, 1);

			return result;
		}
	}
}
